#include <kernel.cuh>

__host__ __device__ unsigned int hash(unsigned int a) {
    a = (a + 0x7ed55d16) + (a << 12);
    a = (a ^ 0xc761c23c) ^ (a >> 19);
    a = (a + 0x165667b1) + (a << 5);
    a = (a + 0xd3a2646c) ^ (a << 9);
    a = (a + 0xfd7046c5) + (a << 3);
    a = (a ^ 0xb55a4f09) ^ (a >> 16);
    return a;
}

__global__ void addKernel(int* a, int* b, int* c)
{
    int tid = threadIdx.x + blockIdx.x * blockDim.x;
    c[tid] = a[tid] + b[tid];
}

__global__ void simple_vbo_kernel(float2* pos, unsigned int size, float speed)
{
    unsigned int x = blockIdx.x * blockDim.x + threadIdx.x;

    if (x < size) {
        // write output vertex
        float2 old_pos = pos[x];
        pos[x] = make_float2(fmod(old_pos.x+ speed +1.,2.)-1, fmod(old_pos.y - speed + 3., 2.) - 1);
    }
}
void addKernelLaunch(int blocksPerGrid, int threadsPerBlock, int* a, int* b, int* c)
{
    addKernel << <blocksPerGrid, threadsPerBlock >> > (a, b, c);
}

void launch_kernel(float2* pos, unsigned int size, float speed)
{
    // execute the kernel
    dim3 block(64, 1);
    dim3 grid(size / block.x+1, 1, 1);
    simple_vbo_kernel << < grid, block >> > (pos, size, speed);
}

void runCuda(struct cudaGraphicsResource** vbo_resource,int points, float speed)
{
    // map OpenGL buffer object for writing from CUDA
    float2* dptr;
    (cudaGraphicsMapResources(1, vbo_resource, 0));
    size_t num_bytes;
    (cudaGraphicsResourceGetMappedPointer((void**)&dptr, &num_bytes,
        *vbo_resource));
    //printf("CUDA mapped VBO: May access %ld bytes\n", num_bytes);

    // execute the kernel
    //    dim3 block(8, 8, 1);
    //    dim3 grid(mesh_width / block.x, mesh_height / block.y, 1);
    //    kernel<<< grid, block>>>(dptr, mesh_width, mesh_height, g_fAnim);

    launch_kernel(dptr, points, speed);

    // unmap buffer object
    (cudaGraphicsUnmapResources(1, vbo_resource, 0));
    cudaDeviceSynchronize();
}

void createVBO(GLuint vbo, struct cudaGraphicsResource** vbo_res)
{

    // create buffer object

    // register this buffer object with CUDA
    (cudaGraphicsGLRegisterBuffer(vbo_res, vbo, cudaGraphicsMapFlagsWriteDiscard));
}
